@@ -1,3 +1,4 @@
+
 # -*- encoding: utf-8 -*-
 #
 # Copyright (c) 2006, 2007 Canonical
@@ -26,18 +27,26 @@ import sys
 import os
 
 from storm.uri import URI
-from storm.expr import Select, Column, Undef, SQLToken, SQLRaw, Count, Alias
+from storm.expr import Select, Column, SQLToken, SQLRaw, Count, Alias
 from storm.variables import (Variable, PickleVariable, RawStrVariable,
                              DecimalVariable, DateTimeVariable, DateVariable,
                              TimeVariable, TimeDeltaVariable)
 from storm.database import *
+from storm.event import EventSystem
 from storm.exceptions import (
-    DatabaseModuleError, DisconnectionError, OperationalError)
+    DatabaseError, DatabaseModuleError, ConnectionBlockedError,
+    DisconnectionError, Error, OperationalError)
 
 from tests.databases.proxy import ProxyTCPServer
 from tests.helper import MakePath
 
 
+class Marker(object):
+    pass
+
+marker = Marker()
+
+
 class DatabaseTest(object):
 
     supports_microseconds = True
@@ -197,7 +206,7 @@ class DatabaseTest(object):
                                          "VALUES ('Title 30')")
         primary_key = (Column("id", SQLToken("test")),
                        Column("title", SQLToken("test")))
-        primary_variables = (Variable(), Variable("Title 30"))
+        primary_variables = (Variable(), Variable(u"Title 30"))
         expr = result.get_insert_identity(primary_key, primary_variables)
         select = Select(Column("title", SQLToken("test")), expr)
         result = self.connection.execute(select)
@@ -336,6 +345,22 @@ class DatabaseTest(object):
         finally:
             connection1.rollback()
 
+    def test_wb_connect_sets_event_system(self):
+        connection = self.database.connect(marker)
+        self.assertEqual(connection._event, marker)
+
+    def test_execute_sends_event(self):
+        event = EventSystem(marker)
+        calls = []
+        def register_transaction(owner):
+            calls.append(owner)
+        event.hook("register-transaction", register_transaction)
+
+        connection = self.database.connect(event)
+        connection.execute("SELECT 1")
+        self.assertEqual(len(calls), 1)
+        self.assertEqual(calls[0], marker)
+
     def from_database(self, row):
         return [int(item)+1 for item in row]
 
@@ -354,6 +379,63 @@ class DatabaseTest(object):
         result.from_database = self.from_database
         self.assertEquals(iter(result).next(), (2, 3))
 
+    def test_rowcount_insert(self):
+        # All supported backends support rowcount, so far.
+        result = self.connection.execute(
+            "INSERT INTO test VALUES (999, '999')")
+        self.assertEquals(result.rowcount, 1)
+
+    def test_rowcount_delete(self):
+        # All supported backends support rowcount, so far.
+        result = self.connection.execute("DELETE FROM test")
+        self.assertEquals(result.rowcount, 2)
+
+    def test_rowcount_update(self):
+        # All supported backends support rowcount, so far.
+        result = self.connection.execute(
+            "UPDATE test SET title='whatever'")
+        self.assertEquals(result.rowcount, 2)
+
+    def test_expr_startswith(self):
+        self.connection.execute("INSERT INTO test VALUES (30, '!!_%blah')")
+        self.connection.execute("INSERT INTO test VALUES (40, '!!blah')")
+        id = Column("id", SQLToken("test"))
+        title = Column("title", SQLToken("test"))
+        expr = Select(id, title.startswith(u"!!_%"))
+        result = list(self.connection.execute(expr))
+        self.assertEquals(result, [(30,)])
+
+    def test_expr_endswith(self):
+        self.connection.execute("INSERT INTO test VALUES (30, 'blah_%!!')")
+        self.connection.execute("INSERT INTO test VALUES (40, 'blah!!')")
+        id = Column("id", SQLToken("test"))
+        title = Column("title", SQLToken("test"))
+        expr = Select(id, title.endswith(u"_%!!"))
+        result = list(self.connection.execute(expr))
+        self.assertEquals(result, [(30,)])
+
+    def test_expr_contains_string(self):
+        self.connection.execute("INSERT INTO test VALUES (30, 'blah_%!!x')")
+        self.connection.execute("INSERT INTO test VALUES (40, 'blah!!x')")
+        id = Column("id", SQLToken("test"))
+        title = Column("title", SQLToken("test"))
+        expr = Select(id, title.contains_string(u"_%!!"))
+        result = list(self.connection.execute(expr))
+        self.assertEquals(result, [(30,)])
+
+    def test_block_access(self):
+        """Access to the connection is blocked by block_access()."""
+        self.connection.execute("SELECT 1")
+        self.connection.block_access()
+        self.assertRaises(ConnectionBlockedError,
+                          self.connection.execute, "SELECT 1")
+        self.assertRaises(ConnectionBlockedError, self.connection.commit)
+        # Allow rolling back a blocked connection.
+        self.connection.rollback()
+        # Unblock the connection, allowing access again.
+        self.connection.unblock_access()
+        self.connection.execute("SELECT 1")
+
 
 class UnsupportedDatabaseTest(object):
     
@@ -492,6 +574,61 @@ class DatabaseDisconnectionTest(object):
         self.proxy.restart()
         self.assertRaises(DisconnectionError, self.connection.commit)
 
+    def test_wb_catch_already_disconnected_on_rollback(self):
+        """Connection.rollback() swallows disconnection errors.
+
+        If the connection is being used outside of Storm's control,
+        then it is possible that Storm won't see the disconnection.
+        It should be able to recover from this situation though.
+        """
+        result = self.connection.execute("SELECT 1")
+        self.assertTrue(result.get_one())
+        self.proxy.restart()
+        # Perform an action that should result in a disconnection.
+        try:
+            cursor = self.connection._raw_connection.cursor()
+            cursor.execute("SELECT 1")
+            cursor.fetchone()
+        except Error, exc:
+            self.assertTrue(self.connection.is_disconnection_error(exc))
+        else:
+            self.fail("Disconnection was not caught.")
+
+        # Make sure our raw connection's rollback() raises a disconnection
+        # error when called.
+        try:
+            self.connection._raw_connection.rollback()
+        except Error, exc:
+            self.assertTrue(self.connection.is_disconnection_error(exc))
+        else:
+            self.fail("Disconnection was not raised.")
+
+        # Our rollback() will catch and swallow that disconnection error,
+        # though.
+        self.connection.rollback()
+
+    def test_wb_catch_already_disconnected(self):
+        """Storm detects connections that have already been disconnected.
+
+        If the connection is being used outside of Storm's control,
+        then it is possible that Storm won't see the disconnection.
+        It should be able to recover from this situation though.
+        """
+        result = self.connection.execute("SELECT 1")
+        self.assertTrue(result.get_one())
+        self.proxy.restart()
+        # Perform an action that should result in a disconnection.
+        try:
+            cursor = self.connection._raw_connection.cursor()
+            cursor.execute("SELECT 1")
+            cursor.fetchone()
+        except DatabaseError, exc:
+            self.assertTrue(self.connection.is_disconnection_error(exc))
+        else:
+            self.fail("Disconnection was not caught.")
+        self.assertRaises(DisconnectionError,
+                          self.connection.execute, "SELECT 1")
+
     def test_connection_stays_disconnected_in_transaction(self):
         """Test that connection does not immediately reconnect."""
         result = self.connection.execute("SELECT 1")
