@@ -184,6 +184,37 @@ class ExprTest(TestHelper):
         expr = Like(elem1, elem2, elem3, False)
         self.assertEquals(expr.case_sensitive, False)
 
+    def test_startswith(self):
+        expr = Func1()
+        self.assertRaises(ExprError, expr.startswith, "not a unicode string")
+
+        like_expr = expr.startswith(u"abc!!_%")
+        self.assertTrue(isinstance(like_expr, Like))
+        self.assertTrue(like_expr.expr1 is expr)
+        self.assertEquals(like_expr.expr2, u"abc!!!!!_!%%")
+        self.assertEquals(like_expr.escape, u"!")
+
+    def test_endswith(self):
+        expr = Func1()
+        self.assertRaises(ExprError, expr.startswith, "not a unicode string")
+
+        like_expr = expr.endswith(u"abc!!_%")
+        self.assertTrue(isinstance(like_expr, Like))
+        self.assertTrue(like_expr.expr1 is expr)
+        self.assertEquals(like_expr.expr2, u"%abc!!!!!_!%")
+        self.assertEquals(like_expr.escape, u"!")
+
+    def test_contains_string(self):
+        expr = Func1()
+        self.assertRaises(
+            ExprError, expr.contains_string, "not a unicode string")
+
+        like_expr = expr.contains_string(u"abc!!_%")
+        self.assertTrue(isinstance(like_expr, Like))
+        self.assertTrue(like_expr.expr1 is expr)
+        self.assertEquals(like_expr.expr2, u"%abc!!!!!_!%%")
+        self.assertEquals(like_expr.escape, u"!")
+
     def test_eq(self):
         expr = Eq(elem1, elem2)
         self.assertEquals(expr.expr1, elem1)
@@ -275,6 +306,35 @@ class ExprTest(TestHelper):
         self.assertEquals(expr.limit, 1)
         self.assertEquals(expr.offset, 2)
 
+    def test_union_collapse(self):
+        expr = Union(Union(elem1, elem2), elem3)
+        self.assertEquals(expr.exprs, (elem1, elem2, elem3))
+
+        # Only first expression is collapsed.
+        expr = Union(elem1, Union(elem2, elem3))
+        self.assertEquals(expr.exprs[0], elem1)
+        self.assertTrue(isinstance(expr.exprs[1], Union))
+
+        # Don't collapse if all is different.
+        expr = Union(Union(elem1, elem2, all=True), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Union))
+        expr = Union(Union(elem1, elem2), elem3, all=True)
+        self.assertTrue(isinstance(expr.exprs[0], Union))
+        expr = Union(Union(elem1, elem2, all=True), elem3, all=True)
+        self.assertEquals(expr.exprs, (elem1, elem2, elem3))
+
+        # Don't collapse if limit or offset are set.
+        expr = Union(Union(elem1, elem2, limit=1), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Union))
+        expr = Union(Union(elem1, elem2, offset=3), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Union))
+
+        # Don't collapse other set expressions.
+        expr = Union(Except(elem1, elem2), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Except))
+        expr = Union(Intersect(elem1, elem2), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Intersect))
+
     def test_except(self):
         expr = Except(elem1, elem2, elem3)
         self.assertEquals(expr.exprs, (elem1, elem2, elem3))
@@ -287,18 +347,77 @@ class ExprTest(TestHelper):
         self.assertEquals(expr.limit, 1)
         self.assertEquals(expr.offset, 2)
 
+    def test_except_collapse(self):
+        expr = Except(Except(elem1, elem2), elem3)
+        self.assertEquals(expr.exprs, (elem1, elem2, elem3))
+
+        # Only first expression is collapsed.
+        expr = Except(elem1, Except(elem2, elem3))
+        self.assertEquals(expr.exprs[0], elem1)
+        self.assertTrue(isinstance(expr.exprs[1], Except))
+
+        # Don't collapse if all is different.
+        expr = Except(Except(elem1, elem2, all=True), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Except))
+        expr = Except(Except(elem1, elem2), elem3, all=True)
+        self.assertTrue(isinstance(expr.exprs[0], Except))
+        expr = Except(Except(elem1, elem2, all=True), elem3, all=True)
+        self.assertEquals(expr.exprs, (elem1, elem2, elem3))
+
+        # Don't collapse if limit or offset are set.
+        expr = Except(Except(elem1, elem2, limit=1), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Except))
+        expr = Except(Except(elem1, elem2, offset=3), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Except))
+
+        # Don't collapse other set expressions.
+        expr = Except(Union(elem1, elem2), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Union))
+        expr = Except(Intersect(elem1, elem2), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Intersect))
+
     def test_intersect(self):
         expr = Intersect(elem1, elem2, elem3)
         self.assertEquals(expr.exprs, (elem1, elem2, elem3))
 
     def test_intersect_with_kwargs(self):
-        expr = Intersect(elem1, elem2, all=True, order_by=(), limit=1, offset=2)
+        expr = Intersect(
+            elem1, elem2, all=True, order_by=(), limit=1, offset=2)
         self.assertEquals(expr.exprs, (elem1, elem2))
         self.assertEquals(expr.all, True)
         self.assertEquals(expr.order_by, ())
         self.assertEquals(expr.limit, 1)
         self.assertEquals(expr.offset, 2)
 
+    def test_intersect_collapse(self):
+        expr = Intersect(Intersect(elem1, elem2), elem3)
+        self.assertEquals(expr.exprs, (elem1, elem2, elem3))
+
+        # Only first expression is collapsed.
+        expr = Intersect(elem1, Intersect(elem2, elem3))
+        self.assertEquals(expr.exprs[0], elem1)
+        self.assertTrue(isinstance(expr.exprs[1], Intersect))
+
+        # Don't collapse if all is different.
+        expr = Intersect(Intersect(elem1, elem2, all=True), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Intersect))
+        expr = Intersect(Intersect(elem1, elem2), elem3, all=True)
+        self.assertTrue(isinstance(expr.exprs[0], Intersect))
+        expr = Intersect(Intersect(elem1, elem2, all=True), elem3, all=True)
+        self.assertEquals(expr.exprs, (elem1, elem2, elem3))
+
+        # Don't collapse if limit or offset are set.
+        expr = Intersect(Intersect(elem1, elem2, limit=1), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Intersect))
+        expr = Intersect(Intersect(elem1, elem2, offset=3), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Intersect))
+
+        # Don't collapse other set expressions.
+        expr = Intersect(Union(elem1, elem2), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Union))
+        expr = Intersect(Except(elem1, elem2), elem3)
+        self.assertTrue(isinstance(expr.exprs[0], Except))
+
     def test_auto_tables(self):
         expr = AutoTables(elem1, [elem2])
         self.assertEquals(expr.expr, elem1)
@@ -450,71 +569,72 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile("str", state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [RawStrVariable("str")])
+        self.assertVariablesEqual(state.parameters, [RawStrVariable("str")])
 
     def test_unicode(self):
         state = State()
         statement = compile(u"str", state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [UnicodeVariable(u"str")])
+        self.assertVariablesEqual(state.parameters, [UnicodeVariable(u"str")])
 
     def test_int(self):
         state = State()
         statement = compile(1, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [IntVariable(1)])
+        self.assertVariablesEqual(state.parameters, [IntVariable(1)])
 
     def test_long(self):
         state = State()
         statement = compile(1L, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [IntVariable(1)])
+        self.assertVariablesEqual(state.parameters, [IntVariable(1)])
 
     def test_bool(self):
         state = State()
         statement = compile(True, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [BoolVariable(1)])
+        self.assertVariablesEqual(state.parameters, [BoolVariable(1)])
 
     def test_float(self):
         state = State()
         statement = compile(1.1, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [FloatVariable(1.1)])
+        self.assertVariablesEqual(state.parameters, [FloatVariable(1.1)])
 
     def test_decimal(self):
         state = State()
         statement = compile(Decimal("1.1"), state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [DecimalVariable(Decimal("1.1"))])
+        self.assertVariablesEqual(
+            state.parameters, [DecimalVariable(Decimal("1.1"))])
 
     def test_datetime(self):
         dt = datetime(1977, 5, 4, 12, 34)
         state = State()
         statement = compile(dt, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [DateTimeVariable(dt)])
+        self.assertVariablesEqual(state.parameters, [DateTimeVariable(dt)])
 
     def test_date(self):
         d = date(1977, 5, 4)
         state = State()
         statement = compile(d, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [DateVariable(d)])
+        self.assertVariablesEqual(state.parameters, [DateVariable(d)])
 
     def test_time(self):
         t = time(12, 34)
         state = State()
         statement = compile(t, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [TimeVariable(t)])
+        self.assertVariablesEqual(state.parameters, [TimeVariable(t)])
 
     def test_timedelta(self):
         td = timedelta(days=1, seconds=2, microseconds=3)
         state = State()
         statement = compile(td, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [TimeDeltaVariable(td)])
+        self.assertVariablesEqual(state.parameters, [TimeDeltaVariable(td)])
 
     def test_none(self):
         state = State()
@@ -574,7 +694,7 @@ class CompileTest(TestHelper):
         self.assertEquals(statement, 'SELECT "table 1".column1 '
                                      'FROM "table 1", "table 2" '
                                      'WHERE "table 2".column2 = ?')
-        self.assertEquals(state.parameters, [Variable(1)])
+        self.assertVariablesEqual(state.parameters, [Variable(1)])
 
     def test_select_auto_table_duplicated(self):
         expr = Select(Column(column1, table1),
@@ -584,7 +704,7 @@ class CompileTest(TestHelper):
         self.assertEquals(statement, 'SELECT "table 1".column1 '
                                      'FROM "table 1" WHERE '
                                      '"table 1".column2 = ?')
-        self.assertEquals(state.parameters, [Variable(1)])
+        self.assertVariablesEqual(state.parameters, [Variable(1)])
 
     def test_select_auto_table_default(self):
         expr = Select(Column(column1),
@@ -594,7 +714,7 @@ class CompileTest(TestHelper):
         statement = compile(expr, state)
         self.assertEquals(statement, 'SELECT column1 FROM "table 1" '
                                      'WHERE column2 = ?')
-        self.assertEquals(state.parameters, [Variable(1)])
+        self.assertVariablesEqual(state.parameters, [Variable(1)])
 
     def test_select_auto_table_default_with_joins(self):
         expr = Select(Column(column1),
@@ -669,7 +789,6 @@ class CompileTest(TestHelper):
                                      'GROUP BY column2 HAVING 1 = 2 '
                                      'ORDER BY column1')
         self.assertEquals(state.parameters, [])
-    
 
     def test_select_contexts(self):
         column, where, table, order_by, group_by = track_contexts(5)
@@ -851,7 +970,7 @@ class CompileTest(TestHelper):
         statement = compile(expr, state)
         self.assertEquals(statement,
                           'DELETE FROM "table 1" WHERE "table 1".column1 = ?')
-        self.assertEquals(state.parameters, [Variable(1)])
+        self.assertVariablesEqual(state.parameters, [Variable(1)])
 
     def test_delete_auto_table_default(self):
         expr = Delete(Column(column1) == 1, default_table=table1)
@@ -859,7 +978,7 @@ class CompileTest(TestHelper):
         statement = compile(expr, state)
         self.assertEquals(statement,
                           'DELETE FROM "table 1" WHERE column1 = ?')
-        self.assertEquals(state.parameters, [Variable(1)])
+        self.assertVariablesEqual(state.parameters, [Variable(1)])
 
     def test_delete_auto_table_unknown(self):
         expr = Delete(Column(column1) == 1)
@@ -904,7 +1023,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_eq(self):
         expr = Eq(Func1(), Func2())
@@ -917,21 +1036,22 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() = ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_is_in(self):
         expr = Func1().is_in(["Hello", "World"])
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() IN (?, ?)")
-        self.assertEquals(state.parameters, [Variable("Hello"), Variable("World")])
+        self.assertVariablesEqual(
+            state.parameters, [Variable("Hello"), Variable("World")])
 
     def test_is_in_empty(self):
         expr = Func1().is_in([])
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "?")
-        self.assertEquals(state.parameters, [BoolVariable(False)])
+        self.assertVariablesEqual(state.parameters, [BoolVariable(False)])
 
     def test_is_in_expr(self):
         expr = Func1().is_in(Select(column1))
@@ -961,7 +1081,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() != ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_ne_none(self):
         expr = Func1() != None
@@ -984,7 +1104,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() > ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_ge(self):
         expr = Ge(Func1(), Func2())
@@ -997,7 +1117,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() >= ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_lt(self):
         expr = Lt(Func1(), Func2())
@@ -1009,7 +1129,7 @@ class CompileTest(TestHelper):
         expr = Func1() < "value"
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() < ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_le(self):
         expr = Le(Func1(), Func2())
@@ -1022,7 +1142,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() <= ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_lshift(self):
         expr = LShift(Func1(), Func2())
@@ -1035,7 +1155,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1()<<?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_rshift(self):
         expr = RShift(Func1(), Func2())
@@ -1048,34 +1168,34 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1()>>?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_like(self):
         expr = Like(Func1(), "value")
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() LIKE ?")
-        self.assertEquals(state.parameters, [RawStrVariable("value")])
+        self.assertVariablesEqual(state.parameters, [RawStrVariable("value")])
 
         expr = Func1().like("Hello")
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() LIKE ?")
-        self.assertEquals(state.parameters, [Variable("Hello")])
+        self.assertVariablesEqual(state.parameters, [Variable("Hello")])
 
     def test_like_escape(self):
         expr = Like(Func1(), "value", "!")
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() LIKE ? ESCAPE ?")
-        self.assertEquals(state.parameters,
+        self.assertVariablesEqual(state.parameters,
                           [RawStrVariable("value"), RawStrVariable("!")])
 
         expr = Func1().like("Hello", "!")
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() LIKE ? ESCAPE ?")
-        self.assertEquals(state.parameters,
+        self.assertVariablesEqual(state.parameters,
                           [Variable("Hello"), RawStrVariable("!")])
 
     def test_like_compareable_case(self):
@@ -1091,7 +1211,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() IN (?)")
-        self.assertEquals(state.parameters, [RawStrVariable("value")])
+        self.assertVariablesEqual(state.parameters, [RawStrVariable("value")])
 
         expr = In(Func1(), elem1)
         state = State()
@@ -1110,7 +1230,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() AND ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_or(self):
         expr = Or(elem1, elem2, Or(elem3, elem4))
@@ -1123,7 +1243,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1() OR ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_and_with_strings(self):
         expr = And("elem1", "elem2")
@@ -1150,7 +1270,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1()+?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_sub(self):
         expr = Sub(elem1, Sub(elem2, elem3))
@@ -1163,13 +1283,13 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "elem1-elem2-elem3")
-        self.assertEquals(state.parameters, [])
+        self.assertVariablesEqual(state.parameters, [])
 
         expr = Func1() - "value"
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1()-?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_mul(self):
         expr = Mul(elem1, elem2, Mul(elem3, elem4))
@@ -1182,7 +1302,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1()*?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_div(self):
         expr = Div(elem1, Div(elem2, elem3))
@@ -1201,7 +1321,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1()/?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_mod(self):
         expr = Mod(elem1, Mod(elem2, elem3))
@@ -1220,7 +1340,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "func1()%?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_func(self):
         expr = Func("myfunc", elem1, Func1(elem2))
@@ -1314,6 +1434,20 @@ class CompileTest(TestHelper):
         self.assertEquals(statement, "UPPER(func1())")
         self.assertEquals(state.parameters, [])
 
+    def test_coalesce(self):
+        expr = Coalesce(Func1())
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "COALESCE(func1())")
+        self.assertEquals(state.parameters, [])
+
+    def test_coalesce_with_many_arguments(self):
+        expr = Coalesce(Func1(), Func2(), None)
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "COALESCE(func1(), func2(), NULL)")
+        self.assertEquals(state.parameters, [])
+
     def test_not(self):
         expr = Not(Func1())
         state = State()
@@ -1328,6 +1462,19 @@ class CompileTest(TestHelper):
         self.assertEquals(statement, "EXISTS func1()")
         self.assertEquals(state.parameters, [])
 
+    def test_neg(self):
+        expr = Neg(Func1())
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "- func1()")
+        self.assertEquals(state.parameters, [])
+
+        expr = -Func1()
+        state = State()
+        statement = compile(expr, state)
+        self.assertEquals(statement, "- func1()")
+        self.assertEquals(state.parameters, [])
+
     def test_asc(self):
         expr = Asc(Func1())
         state = State()
@@ -1477,7 +1624,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "JOIN func1() ON func2() = ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_join_on_with_string(self):
         expr = Join(Func1(), on="a = b")
@@ -1536,7 +1683,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "LEFT JOIN func1() ON func2() = ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_right_join(self):
         expr = RightJoin(Func1())
@@ -1550,7 +1697,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "RIGHT JOIN func1() ON func2() = ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_natural_join(self):
         expr = NaturalJoin(Func1())
@@ -1564,7 +1711,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, "NATURAL JOIN func1() ON func2() = ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_natural_left_join(self):
         expr = NaturalLeftJoin(Func1())
@@ -1579,7 +1726,7 @@ class CompileTest(TestHelper):
         statement = compile(expr, state)
         self.assertEquals(statement, "NATURAL LEFT JOIN func1() "
                                      "ON func2() = ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_natural_right_join(self):
         expr = NaturalRightJoin(Func1())
@@ -1594,7 +1741,7 @@ class CompileTest(TestHelper):
         statement = compile(expr, state)
         self.assertEquals(statement, "NATURAL RIGHT JOIN func1() "
                                      "ON func2() = ?")
-        self.assertEquals(state.parameters, [Variable("value")])
+        self.assertVariablesEqual(state.parameters, [Variable("value")])
 
     def test_union(self):
         expr = Union(Func1(), elem2, elem3)
@@ -1646,9 +1793,10 @@ class CompileTest(TestHelper):
                      order_by=(column1, column2))
         state = State()
         statement = compile(expr, state)
-        self.assertEquals(statement,
-                          '(SELECT elem1 AS "_1") UNION (SELECT elem2 AS "_2") '
-                          'ORDER BY "_1", "_2"')
+        self.assertEquals(
+            statement,
+            '(SELECT elem1 AS "_1") UNION (SELECT elem2 AS "_2") '
+            'ORDER BY "_1", "_2"')
         self.assertEquals(state.parameters, [])
 
     def test_union_contexts(self):
@@ -1716,7 +1864,8 @@ class CompileTest(TestHelper):
         expr = Intersect(Func1(), elem2, elem3, all=True)
         state = State()
         statement = compile(expr, state)
-        self.assertEquals(statement, "func1() INTERSECT ALL elem2 INTERSECT ALL elem3")
+        self.assertEquals(
+            statement, "func1() INTERSECT ALL elem2 INTERSECT ALL elem3")
         self.assertEquals(state.parameters, [])
 
     def test_intersect_order_by_limit_offset(self):
@@ -1735,11 +1884,13 @@ class CompileTest(TestHelper):
         self.assertEquals(state.parameters, [])
 
     def test_intersect_select_nested(self):
-        expr = Intersect(Select(elem1), Intersect(Select(elem2), Select(elem3)))
+        expr = Intersect(
+            Select(elem1), Intersect(Select(elem2), Select(elem3)))
         state = State()
         statement = compile(expr, state)
-        self.assertEquals(statement, "(SELECT elem1) INTERSECT"
-                                     " ((SELECT elem2) INTERSECT (SELECT elem3))")
+        self.assertEquals(
+            statement, "(SELECT elem1) INTERSECT"
+                       " ((SELECT elem2) INTERSECT (SELECT elem3))")
         self.assertEquals(state.parameters, [])
 
     def test_intersect_contexts(self):
@@ -1755,7 +1906,7 @@ class CompileTest(TestHelper):
         state = State()
         statement = compile(expr, state)
         self.assertEquals(statement, 'SELECT ? FROM "table 1"')
-        self.assertEquals(state.parameters, [IntVariable(1)])
+        self.assertVariablesEqual(state.parameters, [IntVariable(1)])
 
     def test_auto_tables_with_column(self):
         expr = Select(AutoTables(Column(elem1, table1), [table2]))
@@ -1815,7 +1966,7 @@ class CompileTest(TestHelper):
                           'SELECT "table 1".elem1 FROM "table 1" JOIN '
                           '"table 2" WHERE ? IN (SELECT "table 1".elem1 '
                           'FROM "table 1" JOIN "table 2")')
-        self.assertEquals(state.parameters, [IntVariable(1)])
+        self.assertVariablesEqual(state.parameters, [IntVariable(1)])
 
     def test_sql_token(self):
         expr = SQLToken("something")
@@ -1931,8 +2082,10 @@ class CompilePythonTest(TestHelper):
 
     def test_compile_sequence(self):
         expr = [elem1, Variable(1), (Variable(2), None)]
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "elem1, 1, 2, None")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "elem1, _0, _1, None")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_compile_invalid(self):
         self.assertRaises(CompileError, compile_python, object())
@@ -1959,8 +2112,10 @@ class CompilePythonTest(TestHelper):
         self.assertEquals(py_expr, "1L")
 
     def test_bool(self):
-        py_expr = compile_python(True)
-        self.assertEquals(py_expr, "True")
+        state = State()
+        py_expr = compile_python(True, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [True])
 
     def test_float(self):
         py_expr = compile_python(1.1)
@@ -1968,23 +2123,31 @@ class CompilePythonTest(TestHelper):
 
     def test_datetime(self):
         dt = datetime(1977, 5, 4, 12, 34)
-        py_expr = compile_python(dt)
-        self.assertEquals(py_expr, repr(dt))
+        state = State()
+        py_expr = compile_python(dt, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [dt])
 
     def test_date(self):
         d = date(1977, 5, 4)
-        py_expr = compile_python(d)
-        self.assertEquals(py_expr, repr(d))
+        state = State()
+        py_expr = compile_python(d, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [d])
 
     def test_time(self):
         t = time(12, 34)
-        py_expr = compile_python(t)
-        self.assertEquals(py_expr, repr(t))
+        state = State()
+        py_expr = compile_python(t, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [t])
 
     def test_timedelta(self):
         td = timedelta(days=1, seconds=2, microseconds=3)
-        py_expr = compile_python(td)
-        self.assertEquals(py_expr, repr(td))
+        state = State()
+        py_expr = compile_python(td, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, [td])
 
     def test_none(self):
         py_expr = compile_python(None)
@@ -1992,63 +2155,87 @@ class CompilePythonTest(TestHelper):
 
     def test_column(self):
         expr = Column(column1)
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "get_column('column1')")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "get_column(_0)")
+        self.assertEquals(state.parameters, [expr])
 
     def test_column_table(self):
         expr = Column(column1, table1)
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "get_column('column1')")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "get_column(_0)")
+        self.assertEquals(state.parameters, [expr])
 
     def test_variable(self):
         expr = Variable("value")
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "'value'")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0")
+        self.assertEquals(state.parameters, ["value"])
 
     def test_eq(self):
         expr = Eq(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 == 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 == _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_ne(self):
         expr = Ne(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 != 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 != _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_gt(self):
         expr = Gt(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 > 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 > _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_ge(self):
         expr = Ge(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 >= 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 >= _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_lt(self):
         expr = Lt(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 < 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 < _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_le(self):
         expr = Le(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 <= 2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 <= _1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_lshift(self):
         expr = LShift(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1<<2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0<<_1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_rshift(self):
         expr = RShift(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1>>2")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0>>_1")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_in(self):
         expr = In(Variable(1), Variable(2))
-        py_expr = compile_python(expr)
-        self.assertEquals(py_expr, "1 in (2,)")
+        state = State()
+        py_expr = compile_python(expr, state)
+        self.assertEquals(py_expr, "_0 in (_1,)")
+        self.assertEquals(state.parameters, [1, 2])
 
     def test_and(self):
         expr = And(elem1, elem2, And(elem3, elem4))
@@ -2065,6 +2252,11 @@ class CompilePythonTest(TestHelper):
         py_expr = compile_python(expr)
         self.assertEquals(py_expr, "elem1+elem2+elem3+elem4")
 
+    def test_neg(self):
+        expr = Neg(elem1)
+        py_expr = compile_python(expr)
+        self.assertEquals(py_expr, "-elem1")
+
     def test_sub(self):
         expr = Sub(elem1, Sub(elem2, elem3))
         py_expr = compile_python(expr)
@@ -2103,8 +2295,20 @@ class CompilePythonTest(TestHelper):
 
         match = compile_python.get_matcher((col1 > 10) & (col2 < 10))
 
-        self.assertTrue(match({"column1": 15, "column2": 5}.get))
-        self.assertFalse(match({"column1": 5, "column2": 15}.get))
+        self.assertTrue(match({col1: 15, col2: 5}.get))
+        self.assertFalse(match({col1: 5, col2: 15}.get))
+
+    def test_match_bad_repr(self):
+        """The get_matcher() works for expressions containing values
+        whose repr is not valid Python syntax."""
+        class BadRepr(object):
+            def __repr__(self):
+                return "$Not a valid Python expression$"
+
+        value = BadRepr()
+        col1 = Column(column1)
+        match = compile_python.get_matcher(col1 == Variable(value))
+        self.assertTrue(match({col1: value}.get))
 
 
 class LazyValueExprTest(TestHelper):
