@@ -21,13 +21,17 @@
 from datetime import datetime, date, time, timedelta
 from decimal import Decimal as decimal
 import gc
+try:
+    import uuid
+except ImportError:
+    uuid = None
 
 from storm.exceptions import NoneError, PropertyPathError
 from storm.properties import PropertyPublisherMeta
 from storm.properties import *
 from storm.variables import *
 from storm.info import get_obj_info
-from storm.expr import Undef, Column, Select, compile, State, SQLRaw
+from storm.expr import Column, Select, compile, State, SQLRaw
 
 from tests.info import Wrapper
 from tests.helper import TestHelper
@@ -255,9 +259,12 @@ class PropertyTest(TestHelper):
                                      "mytable.column1 = ? AND "
                                      "mytable.prop2 = ? AND "
                                      "mytable.column3 = ?")
-        self.assertEquals(state.parameters, [CustomVariable("value1"),
-                                             CustomVariable("value2"),
-                                             CustomVariable("value3")])
+
+        self.assertVariablesEqual(
+            state.parameters,
+            [CustomVariable("value1"),
+             CustomVariable("value2"),
+             CustomVariable("value3")])
 
     def test_comparable_expr_subclass(self):
         prop1 = self.SubClass.prop1
@@ -272,9 +279,11 @@ class PropertyTest(TestHelper):
                                      "mysubtable.column1 = ? AND "
                                      "mysubtable.prop2 = ? AND "
                                      "mysubtable.column3 = ?")
-        self.assertEquals(state.parameters, [CustomVariable("value1"),
-                                             CustomVariable("value2"),
-                                             CustomVariable("value3")])
+        self.assertVariablesEqual(
+            state.parameters,
+            [CustomVariable("value1"),
+             CustomVariable("value2"),
+             CustomVariable("value3")])
 
     def test_set_get_delete_with_wrapper(self):
         obj = self.Class()
@@ -545,6 +554,37 @@ class PropertyKindsTest(TestHelper):
 
         self.assertRaises(TypeError, setattr, self.obj, "prop1", object())
 
+    def test_uuid(self):
+        # Skip test if uuid module is not available.
+        if uuid is None:
+            return
+
+        value1 = uuid.UUID("{0609f76b-878f-4546-baf5-c1b135e8de72}")
+        value2 = uuid.UUID("{c9703f9d-0abb-47d7-a793-8f90f1b98d5e}")
+        self.setup(UUID, default=value1, allow_none=False)
+
+        self.assertTrue(isinstance(self.column1, Column))
+        self.assertTrue(isinstance(self.column2, Column))
+        self.assertEquals(self.column1.name, "column1")
+        self.assertEquals(self.column1.table, self.SubClass)
+        self.assertEquals(self.column2.name, "prop2")
+        self.assertEquals(self.column2.table, self.SubClass)
+        self.assertTrue(isinstance(self.variable1, UUIDVariable))
+        self.assertTrue(isinstance(self.variable2, UUIDVariable))
+
+        self.assertEquals(self.obj.prop1, value1)
+        self.assertRaises(NoneError, setattr, self.obj, "prop1", None)
+        self.obj.prop2 = None
+        self.assertEquals(self.obj.prop2, None)
+
+        self.obj.prop1 = value1
+        self.assertEquals(self.obj.prop1, value1)
+        self.obj.prop1 = value2
+        self.assertEquals(self.obj.prop1, value2)
+
+        self.assertRaises(TypeError, setattr, self.obj, "prop1",
+                          "{0609f76b-878f-4546-baf5-c1b135e8de72}")
+
     def test_enum(self):
         self.setup(Enum, map={"foo": 1, "bar": 2},
                    default="foo", allow_none=False,
@@ -633,6 +673,7 @@ class PropertyKindsTest(TestHelper):
         self.obj.prop2 = []
 
         self.obj_info.checkpoint()
+        self.obj_info.event.emit("start-tracking-changes", self.obj_info.event)
         self.obj_info.event.hook("changed", changed)
 
         self.assertEquals(self.obj.prop1, [])
@@ -682,6 +723,7 @@ class PropertyKindsTest(TestHelper):
             changes.append((variable, old_value, new_value, fromdb))
 
         self.obj_info.checkpoint()
+        self.obj_info.event.emit("start-tracking-changes", self.obj_info.event)
         self.obj_info.event.hook("changed", changed)
 
         self.assertEquals(self.obj.prop1, [])
