@@ -22,19 +22,23 @@ import os
 import gc
 
 from storm.database import create_database
-from storm.properties import Int, List
+from storm.properties import Enum, Int, List
+from storm.info import get_obj_info
 
 from tests.store.base import StoreTest, EmptyResultSetTest, Foo
 from tests.helper import TestHelper
 
-from tests.helper import run_this
-
 
 class Lst1(object):
     __storm_table__ = "lst1"
     id = Int(primary=True)
     ints = List(type=Int())
 
+class LstEnum(object):
+    __storm_table__ = "lst1"
+    id = Int(primary=True)
+    ints = List(type=Enum(map={"one": 1, "two": 2, "three": 3}))
+
 class Lst2(object):
     __storm_table__ = "lst2"
     id = Int(primary=True)
@@ -71,7 +75,7 @@ class PostgresStoreTest(TestHelper, StoreTest):
                            "(id SERIAL PRIMARY KEY,"
                            " foo_id INTEGER, title VARCHAR)")
         connection.execute("CREATE TABLE bin "
-                           "(id SERIAL PRIMARY KEY, bin BYTEA)")
+                           "(id SERIAL PRIMARY KEY, bin BYTEA, foo_id INTEGER)")
         connection.execute("CREATE TABLE link "
                            "(foo_id INTEGER, bar_id INTEGER,"
                            " PRIMARY KEY (foo_id, bar_id))")
@@ -120,6 +124,27 @@ class PostgresStoreTest(TestHelper, StoreTest):
         result = self.store.execute("SELECT ints FROM lst1 WHERE id=1")
         self.assertEquals(result.get_one(), ([1,2,3,4,5],))
 
+    def test_list_enum_variable(self):
+
+        lst = LstEnum()
+        lst.id = 1
+        lst.ints = ["one", "two"]
+        self.store.add(lst)
+
+        result = self.store.execute("SELECT ints FROM lst1 WHERE id=1")
+        self.assertEquals(result.get_one(), ([1,2],))
+
+        del lst
+        gc.collect()
+
+        lst = self.store.find(LstEnum, LstEnum.ints == ["one", "two"]).one()
+        self.assertTrue(lst)
+
+        lst.ints.append("three")
+
+        result = self.store.execute("SELECT ints FROM lst1 WHERE id=1")
+        self.assertEquals(result.get_one(), ([1,2,3],))
+
     def test_list_variable_nested(self):
 
         lst = Lst2()
@@ -163,6 +188,25 @@ class PostgresStoreTest(TestHelper, StoreTest):
         self.store.flush()
         self.assertEquals(foo2.id-foo1.id, 1)
 
+    def test_list_unnecessary_update(self):
+        """
+        Flushing an object with a list variable doesn't create an unnecessary
+        UPDATE statement.
+        """
+        self.store.execute("INSERT INTO lst1 VALUES (1, '{}')", noresult=True)
+
+        lst = self.store.find(Lst1, id=1).one()
+        self.assertTrue(lst)
+        self.store.invalidate()
+
+        lst2 = self.store.find(Lst1, id=1).one()
+        self.assertTrue(lst2)
+        obj_info = get_obj_info(lst2)
+        events = []
+        obj_info.event.hook("changed", lambda *args: events.append(args))
+        self.store.flush()
+        self.assertEquals(events, [])
+
 
 class PostgresEmptyResultSetTest(TestHelper, EmptyResultSetTest):
 
