@@ -18,28 +18,20 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-import thread, weakref, gc
+import threading
+import weakref
+import gc
 
 from tests.helper import TestHelper
+from tests.zope import has_transaction, has_zope_component
 
-try:
+if has_transaction:
     import transaction
-except ImportError:
-    has_transaction = False
-else:
-    has_transaction = True
     from storm.zope.interfaces import IZStorm, ZStormError
     from storm.zope.zstorm import ZStorm, StoreDataManager
 
-try:
+if has_zope_component:
     from zope.component import provideUtility, getUtility
-except ImportError:
-    has_zope_component = False
-else:
-    has_zope_component = True
-
-has_zope = has_transaction and has_zope_component
-
 
 from storm.exceptions import OperationalError
 from storm.locals import Store
@@ -87,18 +79,14 @@ class ZStormTest(TestHelper):
 
         raised = []
 
-        lock = thread.allocate_lock()
-        lock.acquire()
         def f():
             try:
-                try:
-                    self.zstorm.get("name")
-                except ZStormError:
-                    raised.append(True)
-            finally:
-                lock.release()
-        thread.start_new_thread(f, ())
-        lock.acquire()
+                self.zstorm.get("name")
+            except ZStormError:
+                raised.append(True)
+        thread = threading.Thread(target=f)
+        thread.start()
+        thread.join()
 
         self.assertTrue(raised)
 
@@ -228,6 +216,31 @@ class ZStormTest(TestHelper):
         store.execute("SELECT 1")
         self.assertNotInTransaction(store)
 
+    def test_wb_cross_thread_store_does_not_join_transaction(self):
+        """If a zstorm registered thread crosses over to another thread,
+        it will not be usable."""
+        store = self.zstorm.get("name", "sqlite:")
+
+        failures = []
+        def f():
+            # We perform this twice to show that ZStormError is raised
+            # consistently (i.e. not just the first time).
+            for i in range(2):
+                try:
+                    store.execute("SELECT 1")
+                except ZStormError:
+                    failures.append("ZStormError raised")
+                except Exception, exc:
+                    failures.append("Expected ZStormError, got %r" % exc)
+                else:
+                    failures.append("Expected ZStormError, nothing raised")
+                if self._isInTransaction(store):
+                    failures.append("store was joined to transaction")
+        thread = threading.Thread(target=f)
+        thread.start()
+        thread.join()
+        self.assertEqual(failures, ["ZStormError raised"] * 2)
+
     def test_wb_reset(self):
         """_reset is used to reset the zstorm utility between zope test runs.
         """
